//////////////////////////////////////////////////////////////////////////
//                                                                      //
//      This file contains server-sided code for the search events      //
//                                                                      //
//  +----------------------------------------------------------------+  //
//  |                       List of contents                         |  //
//  +----------------------------------------------------------------+  //
//  |----------------------------------------------------------------|  //
//  | 1. Register Content                                            |  //
//  |----------------------------------------------------------------|  //
//  | 2. Output Events                                               |  //
//  |   +------------------------------------------------------------|  //
//  |   | 1. Box Searches                                            |  //
//  |   |------------------------------------------------------------|  //
//  |   | 2. Radius Searches                                         |  //
//  |----------------------------------------------------------------|  //
//  | 3. Input Events                                                |  //
//  |----------------------------------------------------------------|  //
//  | 4. Helper Functions                                            |  //
//  |----------------------------------------------------------------|  //
//  | 5. Search Visualization                                        |  //
//  +----------------------------------------------------------------+  //
//                                                                      //
//////////////////////////////////////////////////////////////////////////
// By Zeblote (1163) //
//////////////////////


///////////////////////////////////////////////////--//
// 1. Register Content                          //--//
// |                                           //--//
////////////////////////////////////////////////--//

registerOutputEvent("fxDTSBrick", "startBoxSearch", "VECTOR\tVECTOR\tLIST All 0 Bricks 1 Players 2 Bots 3 Vehicles 4\tLIST Touching 0 Inside 1", true);
registerOutputEvent("fxDTSBrick", "startRadiusSearch", "VECTOR\tSTRING 100 100\tLIST All 0 Bricks 1 Players 2 Bots 3 Vehicles 4\tLIST Touching 0 Distance 1 Inside 2", true);

registerInputEvent("fxDTSBrick", "onSearchFindBrick", "Self fxDTSBrick\tTargetBrick fxDTSBrick\tPlayer Player\tClient GameConnection\tMinigame Minigame");
registerInputEvent("fxDTSBrick", "onSearchFindPlayer", "Self fxDTSBrick\tTargetPlayer Player\tTargetClient GameConnection\tPlayer Player\tClient GameConnection\tMinigame Minigame");
registerInputEvent("fxDTSBrick", "onSearchFindBot", "Self fxDTSBrick\tTargetBot Bot\tPlayer Player\tClient GameConnection\tMinigame Minigame");
registerInputEvent("fxDTSBrick", "onSearchFindVehicle", "Self fxDTSBrick\tTargetVehicle Vehicle\tPlayer Player\tClient GameConnection\tMinigame Minigame");
registerInputEvent("fxDTSBrick", "onSearchFindNothing", "Self fxDTSBrick\tPlayer Player\tClient GameConnection\tMinigame Minigame");

//There is no need to do file checks like other add-ons do.
//If we have RTB, it is automatically loaded first.
//If we don't have RTB, the base code will set the default values.
RTB_RegisterPref("Max Objects Per Search", "Search Events", "SearchEvents::MaxObjectsPerSearch", "int 1 100000", "Event_Search", 1000, false, false, "");
RTB_RegisterPref("Enable Visualizations", "Search Events", "SearchEvents::VisualizationsEnabled", "bool", "Event_Search", true, false, false, "");
RTB_RegisterPref("Max Box Visualizations", "Search Events", "SearchEvents::MaxBoxVisualizations", "int 1 100", "Event_Search", 12, false, false, "");
RTB_RegisterPref("Max Sphere Visualizations", "Search Events", "SearchEvents::MaxSphereVisualizations", "int 1 100", "Event_Search", 4, false, false, "");
RTB_RegisterPref("Visualization Lifetime", "Search Events", "SearchEvents::VisualizationTime", "int 1 30", "Event_Search", 5, false, false, "");
RTB_RegisterPref("Use Fixed Player Overlap", "Search Events", "SearchEvents::UsePlayerOverlapBox", "bool", "Event_Search", true, false, false, "");
RTB_RegisterPref("Fixed Player Overlap Box", "Search Events", "SearchEvents::PlayerOverlapBox", "string 100", "Event_Search", "-0.7 -0.7 0 0.7 0.7 2.9", false, false, "");

if(!isObject(SearchEvents_Container))
	new ScriptGroup(SearchEvents_Container)
	{
		BoxVisualizations = 0;
		SphereVisualizations = 0;
	};


///////////////////////////////////////////////////--//
// 2. Output Events                             //--//
// |2.1 Box Searches                           //--//
////////////////////////////////////////////////--//

//Perform a box search with a position offset from a brick and a size measured in plates.
function fxDTSBrick::startBoxSearch(%this, %offset, %size, %targetType, %overlapType, %client)
{
	if(isObject(%client))
		%player = %client.player;

	//Convert plates to Tork Units
	%size = SearchEvents::PlatesToTU(%size);
	%offset = SearchEvents::PlatesToTU(%offset);

	//Prepare search
	%position = VectorAdd(setWord(%this.getPosition(), 2, getWord(%this.getWorldBox(), 5)), %offset);
	%box = VectorAdd(%position, VectorScale(%size, -0.5)) SPC VectorAdd(%position, VectorScale(%size, 0.5));

	switch(%targetType)
	{
		case 0: %mask = $TypeMasks::FxBrickAlwaysObjectType | $TypeMasks::PlayerObjectType | $TypeMasks::VehicleObjectType;
		case 1: %mask = $TypeMasks::FxBrickAlwaysObjectType;
		case 2: %mask = $TypeMasks::PlayerObjectType;
		case 3: %mask = $TypeMasks::PlayerObjectType;
		case 4: %mask = $TypeMasks::VehicleObjectType;
	}

	//Init search
	initContainerBoxSearch(%position, %size, %mask);

	//Iterate search results
	%objectsFound = 0;
	%objectsChecked = 0;

	for(%object = containerSearchNext(); isObject(%object); %object = containerSearchNext())
	{
		if(%objectsChecked > $SearchEvents::MaxObjectsPerSearch)
			break;

		%objectsChecked++;

		//Call input functions
		if(%object.getType() & $TypeMasks::FxBrickAlwaysObjectType)
		{
			//Check for full overlap
			if(%overlapType != 1 || (SearchEvents::BoxBoxIntersect(%box, %object.getWorldBox()) == 2))
				%this.onSearchFindBrick(%object, %player, %client);
			else continue;
		}

		else if(%object.getType() & $TypeMasks::PlayerObjectType)
		{
			if((SearchEvents::BoxBoxIntersect(%box, SearchEvents::GetPlayerWorldbox(%object)) - %overlapType) > 0)
			{
				if(%targetType != 3 && !(%object.getClassName() $= "AIPlayer" || !isObject(%object.client)))
					%this.onSearchFindPlayer(%object, %player, %client);
				else if(%targetType != 2 && (%object.getClassName() $= "AIPlayer" || !isObject(%object.client)))
					%this.onSearchFindBot(%object, %player, %client);
				else continue;
			}
			else continue;		
		}

		else if(%object.getType() & $TypeMasks::VehicleObjectType)
		{
			//Check for full overlap
			if(%overlapType != 1 || (SearchEvents::BoxBoxIntersect(%box, %object.getWorldBox()) == 2))
				%this.onSearchFindVehicle(%object, %player, %client);
			else continue;
		}

		%objectsFound++;
	}

	if(%objectsFound == 0)
	{
		%this.onSearchFindNothing(%player, %client);
	}
}


///////////////////////////////////////////////////--//
// 2. Output Events                             //--//
// |2.2 Radius Searches                        //--//
////////////////////////////////////////////////--//

//Perform a radius search with a position offset from a brick and a radius measured in plates.
function fxDTSBrick::startRadiusSearch(%this, %offset, %radius, %targetType, %overlapType, %client)
{
	//Convert plates to Tork Units
	%radius /= 2;
	%offset = SearchEvents::PlatesToTU(%offset);

	//Prepare search
	%position = VectorAdd(setWord(%this.getPosition(), 2, getWord(%this.getWorldBox(), 5)), %offset);

	if(isObject(%client))
		%player = %client.player;

	switch(%targetType)
	{
		case 0: %mask = $TypeMasks::FxBrickAlwaysObjectType | $TypeMasks::PlayerObjectType | $TypeMasks::VehicleObjectType;
		case 1: %mask = $TypeMasks::FxBrickAlwaysObjectType;
		case 2: %mask = $TypeMasks::PlayerObjectType;
		case 3: %mask = $TypeMasks::PlayerObjectType;
		case 4: %mask = $TypeMasks::VehicleObjectType;
	}

	//Init search
	initContainerRadiusSearch(%position, %radius, %mask);

	//Iterate search results
	%objectsFound = 0;
	for(%object = containerSearchNext(); isObject(%object); %object = containerSearchNext())
	{
		if(%objectsFound > $SearchEvents::MaxObjectsPerSearch)
			break;

		//Check whether center is in range
		if(%overlapType != 1 || (containerSearchCurrDist() <= %radius))
		{
			//Call input functions
			if(%object.getType() & $TypeMasks::FxBrickAlwaysObjectType)
			{
				//Check for full overlap
				if(%overlapType != 2 || (SearchEvents::BoxSphereIntersect(%object.getWorldBox(), %position, %radius) == 2))
					%this.onSearchFindBrick(%object, %player, %client);
				else continue;
			}

			else if(%object.getType() & $TypeMasks::PlayerObjectType)
			{
				if((%overlapType == 1)
				|| (%overlapType == 0 && (!$SearchEvents::UsePlayerOverlapBox || SearchEvents::BoxSphereIntersect(SearchEvents::GetPlayerWorldbox(%object), %position, %radius) > 0))
				|| (%overlapType == 2 && (SearchEvents::BoxSphereIntersect(SearchEvents::GetPlayerWorldbox(%object), %position, %radius) > 1)))
				{
					if(%targetType != 3 && !(%object.getClassName() $= "AIPlayer" || !isObject(%object.client)))
						%this.onSearchFindPlayer(%object, %player, %client);
					else if(%targetType != 2 && (%object.getClassName() $= "AIPlayer" || !isObject(%object.client)))
						%this.onSearchFindBot(%object, %player, %client);
					else continue;
				}
				else continue;
			}

			else if(%object.getType() & $TypeMasks::VehicleObjectType)
			{
				//Check for full overlap
				if(%overlapType != 2 || (SearchEvents::BoxSphereIntersect(%object.getWorldBox(), %position, %radius) == 2))
					%this.onSearchFindVehicle(%object, %player, %client);
				else continue;
			}
		}
		else continue;

		%objectsFound++;
	}

	if(%objectsFound == 0)
	{

		%this.onSearchFindNothing(%player, %client);
	}
}


///////////////////////////////////////////////////--//
// 3. Input Functions                           //--//
// |                                           //--//
////////////////////////////////////////////////--//

//Found a brick
function fxDTSBrick::onSearchFindBrick(%this, %object, %player, %client)
{
	//Trust Check
	if(getTrustLevel(%object,%client) < 2) {
		return;
	}
	
	//Setup input targets
	$InputTarget_["Self"] = %this;
	$InputTarget_["TargetBrick"] = %object;
	$InputTarget_["Player"] = %player;
	$InputTarget_["Client"] = %client;
	$InputTarget_["Minigame"] = SearchEvents::GetMinigameForInput(%this, %object);

	//Process event
	%this.processInputEvent("onSearchFindBrick", %client);
}

//Found a player
function fxDTSBrick::onSearchFindPlayer(%this, %object, %player, %client)
{
	//Setup input targets
	$InputTarget_["Self"] = %this;
	$InputTarget_["TargetPlayer"] = %object;
	$InputTarget_["TargetClient"] = %object.client;
	$InputTarget_["Player"] = %player;
	$InputTarget_["Client"] = %client;
	$InputTarget_["Minigame"] = SearchEvents::GetMinigameForInput(%this, %object);

	//Process event
	%this.processInputEvent("onSearchFindPlayer", %object.client);	
}

//Found a bot
function fxDTSBrick::onSearchFindBot(%this, %object, %player, %client)
{
	//Setup input targets
	$InputTarget_["Self"] = %this;
	$InputTarget_["TargetBot"] = %object;
	$InputTarget_["Player"] = %player;
	$InputTarget_["Client"] = %client;
	$InputTarget_["Minigame"] = SearchEvents::GetMinigameForInput(%this, %object);

	//Process event
	%this.processInputEvent("onSearchFindBot", %client);
}

//Found a vehicle
function fxDTSBrick::onSearchFindVehicle(%this, %object, %player, %client)
{
	//Setup input targets
	$InputTarget_["Self"] = %this;
	$InputTarget_["TargetVehicle"] = %object;
	$InputTarget_["Player"] = %player;
	$InputTarget_["Client"] = %client;
	$InputTarget_["Minigame"] = SearchEvents::GetMinigameForInput(%this, %object);

	//Process event
	%this.processInputEvent("onSearchFindVehicle", %client);
}

//Found nothing
function fxDTSBrick::onSearchFindNothing(%this, %player, %client)
{
	//Setup input targets
	$InputTarget_["Self"] = %this;
	$InputTarget_["Player"] = %player;
	$InputTarget_["Client"] = %client;
	$InputTarget_["Minigame"] = SearchEvents::GetMinigameForInput(%this, %object);

	//Process event
	%this.processInputEvent("onSearchFindNothing", %client);
}


///////////////////////////////////////////////////--//
// 4. Helper Functions                          //--//
// |                                           //--//
////////////////////////////////////////////////--//

//Find a suitable minigame target
function SearchEvents::GetMinigameForInput(%brick, %object)
{
	if($Server::LAN)
		return getMiniGameFromObject(%object);
	if(getMiniGameFromObject(%brick) == getMiniGameFromObject(%object))
		return getMiniGameFromObject(%brick);
	return 0;
}

//Cnvert plates to Tork Units
function SearchEvents::PlatesToTU(%vector)
{
	%x = getWord(%vector, 0) / 2;
	%y = getWord(%vector, 1) / 2;
	%z = getWord(%vector, 2) / 5;

	return %x SPC %y SPC %z;
}

//Cnvert Tork Units to plates
function SearchEvents::TUToPlates(%vector)
{
	%x = getWord(%vector, 0) * 2;
	%y = getWord(%vector, 1) * 2;
	%z = getWord(%vector, 2) * 5;

	return %x SPC %y SPC %z;
}

//Check whether two world boxes overlap
function SearchEvents::BoxBoxIntersect(%box1, %box2)
{
	%x1 = getWord(%box1, 0);
	%y1 = getWord(%box1, 1);
	%z1 = getWord(%box1, 2);
	%x12 = getWord(%box1, 3);
	%y12 = getWord(%box1, 4);
	%z12 = getWord(%box1, 5);

	%x2 = getWord(%box2, 0);
	%y2 = getWord(%box2, 1);
	%z2 = getWord(%box2, 2);
	%x22 = getWord(%box2, 3);
	%y22 = getWord(%box2, 4);
	%z22 = getWord(%box2, 5);

	if((%x1 > %x22 || %x2 > %x12)
	|| (%y1 > %y22 || %y2 > %y12)
	|| (%z1 > %z22 || %z2 > %z12))
		return 0;

	if((%x2 >= %x1 && %x22 <= %x12)
	&& (%y2 >= %y1 && %y22 <= %y12)
	&& (%z2 >= %z1 && %z22 <= %z12))
		return 2;

	return 1;
}

//Check whether sphere and world box overlap
//We simply check whether the points are inside the sphere
//As players are generally smaller than the search sphere, this sort of works
function SearchEvents::BoxSphereIntersect(%box1, %center, %radius)
{
	%x1 = getWord(%box1, 0);
	%y1 = getWord(%box1, 1);
	%z1 = getWord(%box1, 2);
	%x2 = getWord(%box1, 3);
	%y2 = getWord(%box1, 4);
	%z2 = getWord(%box1, 5);

	%matches = 0;
	%matches += (VectorDist(%center, %x1 SPC %y1 SPC %z1) <= %radius);
	%matches += (VectorDist(%center, %x1 SPC %y2 SPC %z1) <= %radius);
	%matches += (VectorDist(%center, %x1 SPC %y1 SPC %z2) <= %radius);
	%matches += (VectorDist(%center, %x1 SPC %y2 SPC %z2) <= %radius);
	%matches += (VectorDist(%center, %x2 SPC %y1 SPC %z1) <= %radius);
	%matches += (VectorDist(%center, %x2 SPC %y2 SPC %z1) <= %radius);
	%matches += (VectorDist(%center, %x2 SPC %y1 SPC %z2) <= %radius);
	%matches += (VectorDist(%center, %x2 SPC %y2 SPC %z2) <= %radius);

	if(%matches < 1)
		return 0;
	else if(%matches > 7)
		return 2;
	else
		return 1;
}

//Get a proper world box for a player, default one is broken
function SearchEvents::GetPlayerWorldbox(%player)
{
	if(!$SearchEvents::UsePlayerOverlapBox || %player.getDatablock().shapeFile !$= "base/data/shapes/player/m.dts")
		return %player.getWorldBox();

	%lower = VectorScale(getWords($SearchEvents::PlayerOverlapBox, 0, 2), %player.getScale());
	%upper = VectorScale(getWords($SearchEvents::PlayerOverlapBox, 3, 5), %player.getScale());

	//Crouched player isn't as tall
	if(%player.isCrouched())
		%upper = setWord(%upper, 2, getWord(%upper, 2) / 2.5);

	//Get box based on offset and position
	return VectorAdd(%player.getPosition(), %lower) SPC VectorAdd(%player.getPosition(), %upper);
}


///////////////////////////////////////////////////--//
// 5. Search Visualization                      //--//
// |                                           //--//
////////////////////////////////////////////////--//

//Datablock for search visualization
if(!isObject(SearchEvents_LaserShape))
	datablock StaticShapeData(SearchEvents_LaserShape)
	{
		shapeFile = "./laser6.dts";
	};

//Creates a line between 2 points
function SearchEvents::DebugLine(%vec1, %vec2, %container)
{
	%diff = VectorSub(%vec1, %vec2);
	%norm = VectorNormalize(%diff);

	%laser = new StaticShape()
	{
		datablock = SearchEvents_LaserShape;
		scale = VectorLen(%diff) * 5 SPC "0.5 0.5";
		position = VectorAdd(%vec2, VectorScale(%diff, 0.5));
		rotation = VectorNormalize(VectorCross("1 0 0", %norm)) SPC mRadToDeg(mACos(VectorDot("1 0 0", %norm))) * -1;
	};

	%laser.setNodeColor("ALL", "0 0 0.5 0.5");

	if(isObject(%container))
		%container.add(%laser);
}

//Creates a box visualization (12 shapes)
function SearchEvents::DebugBox(%position, %size)
{
	if(SearchEvents_Container.BoxVisualizations >= $SearchEvents::MaxBoxVisualizations)
		return 0;

	SearchEvents_Container.add(
		%container = new ScriptGroup(SearchEvents_BoxVisualization){}
	);
	SearchEvents_Container.BoxVisualizations++;

	//Get the 8 points of the box	
	%x = getWord(%size, 0) * 0.5;
	%y = getWord(%size, 1) * 0.5;
	%z = getWord(%size, 2) * 0.5;

	%p1 = VectorAdd(%position, %x SPC %y SPC %z);
	%p2 = VectorAdd(%position, -%x SPC %y SPC %z);
	%p3 = VectorAdd(%position, %x SPC -%y SPC %z);
	%p4 = VectorAdd(%position, -%x SPC -%y SPC %z);

	%p5 = VectorAdd(%position, -%x SPC -%y SPC -%z);
	%p6 = VectorAdd(%position, %x SPC -%y SPC -%z);
	%p7 = VectorAdd(%position, -%x SPC %y SPC -%z);
	%p8 = VectorAdd(%position, %x SPC %y SPC -%z);

	//Create lasers between the points
	SearchEvents::DebugLine(%p1, %p8, %container);
	SearchEvents::DebugLine(%p2, %p7, %container);
	SearchEvents::DebugLine(%p3, %p6, %container);
	SearchEvents::DebugLine(%p4, %p5, %container);

	SearchEvents::DebugLine(%p1, %p2, %container);
	SearchEvents::DebugLine(%p2, %p4, %container);
	SearchEvents::DebugLine(%p4, %p3, %container);
	SearchEvents::DebugLine(%p3, %p1, %container);

	SearchEvents::DebugLine(%p5, %p6, %container);
	SearchEvents::DebugLine(%p6, %p8, %container);
	SearchEvents::DebugLine(%p8, %p7, %container);
	SearchEvents::DebugLine(%p7, %p5, %container);

	//These must be deleted at all costs. Don't apply schedule quota.
	%container.scheduleNoQuota($SearchEvents::VisualizationTime * 1000, "TimeOut");

	return 1;
}

//Creates a sphere visualization (36 shapes)
function SearchEvents::DebugSphere(%position, %radius)
{
	if(SearchEvents_Container.SphereVisualizations >= $SearchEvents::MaxSphereVisualizations)
		return 0;

	SearchEvents_Container.add(
		%container = new ScriptGroup(SearchEvents_SphereVisualization){}
	);
	SearchEvents_Container.SphereVisualizations++;

	//top
	%p1 = VectorAdd(%position, "0 0 " @ %radius);
	//bottom
	%p2 = VectorAdd(%position, "0 0 " @ (%radius * -1));
	//right
	%p3 = VectorAdd(%position, "0 " @ %radius @ " 0");
	//left
	%p4 = VectorAdd(%position, "0 " @ (%radius * -1) @ " 0");
	//front
	%p5 = VectorAdd(%position, %radius @ " 0 0");
	//back
	%p6 = VectorAdd(%position, (%radius * -1) @ " 0 0");

	//Cross
	//SearchEvents::DebugLine(%p1, %p2);
	//SearchEvents::DebugLine(%p3, %p4);
	//SearchEvents::DebugLine(%p5, %p6);

	//Circles
	%cos30 = mCos(mDegToRad(30));
	%sin30 = mSin(mDegToRad(30));
	%cos60 = mCos(mDegToRad(60));
	%sin60 = mSin(mDegToRad(60));

	%x30 = -%radius * %sin30;
	%y30 = %radius * %cos30;
	%x60 = -%radius * %sin60;
	%y60 = %radius * %cos60;

	//1 of 3
	%pos1 = VectorAdd(0 SPC %x30 SPC %y30, %position);
	%pos2 = VectorAdd(0 SPC %x60 SPC %y60, %position);
	SearchEvents::DebugLine(%p1, %pos1, %container);
	SearchEvents::DebugLine(%pos1, %pos2, %container);
	SearchEvents::DebugLine(%pos2, %p4, %container);

	%pos1 = VectorAdd(0 SPC %x30 SPC -%y30, %position);
	%pos2 = VectorAdd(0 SPC %x60 SPC -%y60, %position);
	SearchEvents::DebugLine(%p2, %pos1, %container);
	SearchEvents::DebugLine(%pos1, %pos2, %container);
	SearchEvents::DebugLine(%pos2, %p4, %container);

	%pos1 = VectorAdd(0 SPC -%x30 SPC %y30, %position);
	%pos2 = VectorAdd(0 SPC -%x60 SPC %y60, %position);
	SearchEvents::DebugLine(%p1, %pos1, %container);
	SearchEvents::DebugLine(%pos1, %pos2, %container);
	SearchEvents::DebugLine(%pos2, %p3, %container);

	%pos1 = VectorAdd(0 SPC -%x30 SPC -%y30, %position);
	%pos2 = VectorAdd(0 SPC -%x60 SPC -%y60, %position);
	SearchEvents::DebugLine(%p2, %pos1, %container);
	SearchEvents::DebugLine(%pos1, %pos2, %container);
	SearchEvents::DebugLine(%pos2, %p3, %container);

	//2 of 3
	%pos1 = VectorAdd(%x30 SPC 0 SPC %y30, %position);
	%pos2 = VectorAdd(%x60 SPC 0 SPC  %y60, %position);
	SearchEvents::DebugLine(%p1, %pos1, %container);
	SearchEvents::DebugLine(%pos1, %pos2, %container);
	SearchEvents::DebugLine(%pos2, %p6, %container);

	%pos1 = VectorAdd(%x30 SPC 0 SPC -%y30, %position);
	%pos2 = VectorAdd(%x60 SPC 0 SPC -%y60, %position);
	SearchEvents::DebugLine(%p2, %pos1, %container);
	SearchEvents::DebugLine(%pos1, %pos2, %container);
	SearchEvents::DebugLine(%pos2, %p6, %container);

	%pos1 = VectorAdd(-%x30 SPC 0 SPC %y30, %position);
	%pos2 = VectorAdd(-%x60 SPC 0 SPC %y60, %position);
	SearchEvents::DebugLine(%p1, %pos1, %container);
	SearchEvents::DebugLine(%pos1, %pos2, %container);
	SearchEvents::DebugLine(%pos2, %p5, %container);

	%pos1 = VectorAdd(-%x30 SPC 0 SPC -%y30, %position);
	%pos2 = VectorAdd(-%x60 SPC 0 SPC -%y60, %position);
	SearchEvents::DebugLine(%p2, %pos1, %container);
	SearchEvents::DebugLine(%pos1, %pos2, %container);
	SearchEvents::DebugLine(%pos2, %p5, %container);

	//3 of 3
	%pos1 = VectorAdd(%x30 SPC %y30 SPC 0, %position);
	%pos2 = VectorAdd(%x60 SPC %y60 SPC 0, %position);
	SearchEvents::DebugLine(%p3, %pos1, %container);
	SearchEvents::DebugLine(%pos1, %pos2, %container);
	SearchEvents::DebugLine(%pos2, %p6, %container);

	%pos1 = VectorAdd(%x30 SPC -%y30 SPC 0, %position);
	%pos2 = VectorAdd(%x60 SPC -%y60 SPC 0, %position);
	SearchEvents::DebugLine(%p4, %pos1, %container);
	SearchEvents::DebugLine(%pos1, %pos2, %container);
	SearchEvents::DebugLine(%pos2, %p6, %container);

	%pos1 = VectorAdd(-%x30 SPC %y30 SPC 0, %position);
	%pos2 = VectorAdd(-%x60 SPC %y60 SPC 0, %position);
	SearchEvents::DebugLine(%p3, %pos1, %container);
	SearchEvents::DebugLine(%pos1, %pos2, %container);
	SearchEvents::DebugLine(%pos2, %p5, %container);

	%pos1 = VectorAdd(-%x30 SPC -%y30 SPC 0, %position);
	%pos2 = VectorAdd(-%x60 SPC -%y60 SPC 0, %position);
	SearchEvents::DebugLine(%p4, %pos1, %container);
	SearchEvents::DebugLine(%pos1, %pos2, %container);
	SearchEvents::DebugLine(%pos2, %p5, %container);

	//These must be deleted at all costs. Don't apply schedule quota.
	%container.scheduleNoQuota($SearchEvents::VisualizationTime * 1000, "TimeOut");

	return 1;
}

//Removes a box visualization and makes room for more
function SearchEvents_BoxVisualization::TimeOut(%container)
{
	%container.delete();
	SearchEvents_Container.BoxVisualizations--;
}

//Removes a sphere visualization and makes room for more
function SearchEvents_SphereVisualization::TimeOut(%container)
{
	%container.delete();
	SearchEvents_Container.SphereVisualizations--;
}

package SearchEvents
{
	//Visualize searches when event is added
	function serverCmdAddEvent(%client, %enabled, %inputEventIdx, %delay, %targetIdx, %NTNameIdx, %outputEventIdx, %par1, %par2, %par3, %par4)
	{
		if(outputEvent_GetOutputName("fxDTSBrick", %outputEventIdx) $= "startBoxSearch" && isObject(%client.wrenchBrick))
		{
			if(%par2 $= "0 0 0")
			{
				if(%targetIdx == -1)
				{
					%group = %client.wrenchBrick.getGroup();
					%nt = %group.NTName[%NTNameIdx];
					%brick = %group.NTObject[%nt, 0];
				}
				else
					%brick = %client.wrenchBrick;

				%wb = %brick.getWorldBox();
				%par2 = VectorSub(getWords(%wb, 3, 5), getWords(%wb, 0, 2));
				%par2 = SearchEvents::TUToPlates(%par2);
				%par1 = "0 0" SPC getWord(%par2, 2) / 2;
			}

			if($SearchEvents::VisualizationsEnabled)
			{
				//Convert plates to Tork Units
				%size = SearchEvents::PlatesToTU(%par2);
				%offset = SearchEvents::PlatesToTU(%par1);

				if(%targetIdx == -1)
				{
					//Show debug box for all named bricks
					%group = %client.wrenchBrick.getGroup();
					%nt = %group.NTName[%NTNameIdx];
					for(%i = 0; %i < %group.NTObjectCount[%nt]; %i++)
					{
						%brick = %group.NTObject[%nt, %i];
						%position = VectorAdd(setWord(%brick.getPosition(), 2, getWord(%brick.getWorldBox(), 5)), %offset);

						if(!SearchEvents::DebugBox(%position, %size))
							commandToClient(%client, 'centerPrint', "\c6One or more searches could not be previewed as you hit the limit.", 5);
					}
				}
				else
				{
					//No named brick, show on this brick
					%brick = %client.wrenchBrick;
					%position = VectorAdd(setWord(%brick.getPosition(), 2, getWord(%brick.getWorldBox(), 5)), %offset);

					if(!SearchEvents::DebugBox(%position, %size))
						commandToClient(%client, 'centerPrint', "\c6One or more searches could not be previewed as you hit the limit.", 5);
				}
			}
		}
		else if(outputEvent_GetOutputName("fxDTSBrick", %outputEventIdx) $= "startRadiusSearch" && isObject(%client.wrenchBrick))
		{
			if(%par2 <= 0)
			{
				if(%targetIdx == -1)
				{
					%group = %client.wrenchBrick.getGroup();
					%nt = %group.NTName[%NTNameIdx];
					%brick = %group.NTObject[%nt, 0];
				}
				else
					%brick = %client.wrenchBrick;

				%wb = %brick.getWorldBox();
				%par2 = mMin(getWord(%wb, 3) - getWord(%wb, 0), getWord(%wb, 4) - getWord(%wb, 1));
				%par1 = "0 0 0";
			}

			if($SearchEvents::VisualizationsEnabled)
			{
				//Convert plates to Tork Units
				%radius = %par2 / 2;
				%offset = SearchEvents::PlatesToTU(%par1);

				if(%targetIdx == -1)							
				{
					//Show debug box for all named bricks
					%group = %client.wrenchBrick.getGroup();
					%nt = %group.NTName[%NTNameIdx];

					for(%i = 0; %i < %group.NTObjectCount[%nt]; %i++)
					{
						%brick = %group.NTObject[%nt, %i];
						%position = VectorAdd(setWord(%brick.getPosition(), 2, getWord(%brick.getWorldBox(), 5)), %offset);

						if(!SearchEvents::DebugSphere(%position, %radius))
							commandToClient(%client, 'centerPrint', "\c6One or more searches could not be previewed as you hit the limit.", 5);
					}
				}
				else
				{
					//No named brick, show on this brick
					%brick = %client.wrenchBrick;
					%position = VectorAdd(setWord(%brick.getPosition(), 2, getWord(%brick.getWorldBox(), 5)), %offset);

					if(!SearchEvents::DebugSphere(%position, %radius))
						commandToClient(%client, 'centerPrint', "\c6One or more searches could not be previewed as you hit the limit.", 5);
				}
			}
		}

		parent::serverCmdAddEvent(%client, %enabled, %inputEventIdx, %delay, %targetIdx, %NTNameIdx, %outputEventIdx, %par1, %par2, %par3, %par4);
	}
};
activatePackage(SearchEvents);